﻿using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.SearchModels;
using SecuritySystemContracts.StoragesContracts;
using SecuritySystemContracts.ViewModels;
using SecuritySystemDataModels.Models;
using SecuritySystemFileImplement.Models;

namespace SecuritySystemFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton source;
        public ShopStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            return source.Shops.FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name))
            {
                return new();
            }
            return source.Shops
                .Select(x => x.GetViewModel)
                .Where(x => x.Name.Contains(model.Name ?? string.Empty))
                .ToList();
        }

        public List<ShopViewModel> GetFullList()
        {
            return source.Shops.Select(shop => shop.GetViewModel).ToList();
        }

        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = source.Shops.Count > 0 ? source.Shops.Max(x => x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            source.Shops.Add(newShop);
            source.SaveShops();
            return newShop.GetViewModel;
        }

        public ShopViewModel? Update(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }
            shop.Update(model);
            source.SaveShops();
            return shop.GetViewModel;
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var shop = source.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }
            source.Shops.Remove(shop);
            source.SaveShops();
            return shop.GetViewModel;
        }

        public bool SellSecures(ISecureModel secureModel, int securesCount)
        {
            var secure = source.Secures.FirstOrDefault(x => x.Id == secureModel.Id);

            if (secure == null)
            {
                return false;
            }

            var shopSecures = source.Shops.SelectMany(shop => shop.ShopSecures.Where(c => c.Value.Item1.Id == secure.Id));

            if (!CanSellSecures(secureModel, securesCount))
            {
                return false;
            }

            foreach (var shop in source.Shops)
            {
                var secures = shop.ShopSecures;

                foreach (var c in secures.Where(x => x.Value.Item1.Id == secure.Id))
                {
                    int min = Math.Min(c.Value.Item2, securesCount);
                    secures[c.Value.Item1.Id] = (c.Value.Item1, c.Value.Item2 - min);
                    securesCount -= min;

                    if (securesCount <= 0)
                        break;
                }

                shop.Update(new ShopBindingModel
                {
                    Id = shop.Id,
                    Name = shop.Name,
                    Address = shop.Address,
                    MaxSecuresCount = shop.MaxSecuresCount,
                    OpeningDate = shop.OpeningDate,
                    ShopSecures = secures
                });

                source.SaveShops();

                if (securesCount <= 0)
                    return true;
            }

            return true;
        }


        private int GetSecuresCount(ISecureModel secureModel)
        {
            var shopSecures = source.Shops.SelectMany(shop => shop.ShopSecures.Where(c => c.Value.Item1.Id == secureModel.Id));
            // посчитаем количество изделий во всех магазинах
            return shopSecures.Select(x => x.Value.Item2).Sum();
        }

        public bool CanSellSecures(ISecureModel secureModel, int securesCount)
        {
            return GetSecuresCount(secureModel) >= securesCount;
        }
    }
}
